function [Mats]=NoGapReMats(x,isObsMats,isEvalGensys,fid,varargin)

% Options:
%   1. [Mats]=NoGapReMats(x)
%   2. [Mats]=NoGapReMats(x,isObsMats)
%   3. [Mats]=NoGapReMats(x,isObsMats,isEvalGensys)
%   4. [Mats]=NoGapReMats(x,isObsMats,isEvalGensys,fid)
%   5. [Mats]=NoGapReMats(...,varargin)
%   Default: Option 1 where Mats is a structure with fields 'ObsEq','StateEq', and 'REE'
%
% Inputs:
%   x
%   Vector of parameters
%
%   isObsMats (Optional)
%   Indicates whether to output Observable matrices and/or Gensys matrices.
%   0: No observable matrices.
%   1: Function outputs observable matrices ONLY--no gensys matrices.
%   2: Function outputs observable matrices and gensys matrices, according to 'isEvalGensys' options.
%   Default: 2
%
%   isEvalGensys (Optional)
%   Flags whether to evalute gensys and determines what function will output from gensys.
%   0: Do not evaluate gensys. Function outputs gensys input matrices only.
%   1: Evalute gensys. Function outputs gensys output matrices only.
%   2: Evalute gensys. Function outputs both gensys input and output matrices.
%   Default: 1
%
%   varargin (optional)
%   Allows for arguments for verbose and flie id to pass to gensys if needed.
% Outputs:
%   Mats is a structure with fields dependent on the values of the input flags:
%      ObsEq.HBar,ObsEq.H
%          Transition matrices from observable equation
%
%      REE.GBar,REE.G1,REE.G2,REE.eu
%          Gensys output matrices, including the error code vector eu.
%
%      StateEq.Gamma0,StateEq.Gamma1,StateEq.GammaBar,StateEq.Gamma2,StateEq.Gamma3
%          Gensys inputs
%
% Gensys requires the following form:
%   Gamma0*StateVar_t = GammaBar + Gamma1*StateVar_t_1 + Gamma2*ShockVar_t + Gamma3*eta_t
% and yields system in form:
%   StateVar_t = GBar + G1*StateVar_t_1 + G2*ShockVar_t
%
% The observation equations are given by:
%   ObsVar_t = HBar + H*StateVar_t
%
% Created: 2013/11/15 with MakeMats.m 

%% Set default input values
if nargin<2, isObsMats=2; end
if nargin<3, isEvalGensys=1; end
if nargin<4, fid=1; end

%% Map parameters
JPTalpha = x(1);
JPTiotap = x(2);
JPTiotaw = x(3);
JPT100gamma = x(4);
JPTh = x(5);
JPTlambdap = x(6);
JPTlogL = x(7);
JPT100pi = x(8);
JPT100DiscRate = x(9);
JPTnu = x(10);
JPTxip = x(11);
JPTxiw = x(12);
JPTchi = x(13);
JPTSdd = x(14);
JPTphipi = x(15);
JPTrhoR = x(16);
JPTrhoz = x(17);
JPTrhog = x(18);
JPTrhomu = x(19);
JPTrhop = x(20);
JPTrhow = x(21);
JPTrhob = x(22);
JPTthetap = x(23);
JPTthetaw = x(24);
JPTsigmamp = x(25);
JPTsigmaz = x(26);
JPTsigmag = x(27);
JPTsigmamu = x(28);
JPTsigmap = x(29);
JPTsigmaw = x(30);
JPTsigmab = x(31);

%% Create Gensys Inputs
GammaBar = [...
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    0;
    ];

Gamma0 = [...
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, -1/((JPT100DiscRate/100 + 1)*(JPTiotap/(JPT100DiscRate/100 + 1) + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, (JPTh*exp(JPT100gamma/100))/((JPT100DiscRate/100 + 1)*(JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, (JPTh*exp(JPT100gamma/100))/((JPT100DiscRate/100 + 1)*(JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1, 0, 0, 0, 39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1, 0, 0, 0, -39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1, 0, 0, 39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1, 0, 0, 0, -39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -(JPTSdd*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -(JPTSdd*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, -1/((JPT100DiscRate/100 + 1)*(1/(JPT100DiscRate/100 + 1) + 1)), -1/((JPT100DiscRate/100 + 1)*(1/(JPT100DiscRate/100 + 1) + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    ];

Gamma1 = [...
    -1, JPTalpha*(((-JPTalpha/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))^JPTalpha - 1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) + JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))) + 1), -(((-JPTalpha/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))^JPTalpha - 1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) + JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))) + 1)*(JPTalpha - 1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, JPTalpha*(((-JPTalpha/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))^JPTalpha - 1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) + JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))) + 1), -(((-JPTalpha/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))^JPTalpha - 1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) + JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))) + 1)*(JPTalpha - 1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, -1, 1, -1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 1, -1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, JPTalpha, 1 - JPTalpha, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTalpha, 1 - JPTalpha, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, -1, ((JPTxip/(JPT100DiscRate/100 + 1) - 1)*(JPTxip - 1))/(JPTxip*(JPTiotap/(JPT100DiscRate/100 + 1) + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTiotap/(JPTiotap/(JPT100DiscRate/100 + 1) + 1), 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, ((JPTxip/(JPT100DiscRate/100 + 1) - 1)*(JPTxip - 1))/(JPTxip*(JPTiotap/(JPT100DiscRate/100 + 1) + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, -1, (JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0, 0, (JPTh*exp(JPT100gamma/100) - (JPTh*JPTrhoz*exp(JPT100gamma/100))/(JPT100DiscRate/100 + 1))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, (JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/100)*JPTh + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))*(JPTrhob - 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -(JPTh*exp(JPT100gamma/100))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, (JPTh*exp(JPT100gamma/100) - (JPTh*JPTrhoz*exp(JPT100gamma/100))/(JPT100DiscRate/100 + 1))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, (JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/100)*JPTh + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))*(JPTrhob - 1)), 0, 0, 0, 0, 0, 0, 0, -1, (JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -(JPTh*exp(JPT100gamma/100))/((JPTh - exp(JPT100gamma/100))*(exp(JPT100gamma/100) - JPTh/(JPT100DiscRate/100 + 1))), 0, 0;
    0, 0, 0, 0, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 0, 0, -JPTrhoz, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -JPTrhoz, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 1/JPTchi, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1/JPTchi, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, JPTrhoz*(39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1) - (39*JPTrhoz)/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhoz*(39/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)) - 1) - (39*JPTrhoz)/(40*exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 1, - JPTSdd*exp(JPT100gamma/50) - (JPTSdd*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1), 0, 0, 0, (JPTSdd*JPTrhoz*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1) - JPTSdd*exp(JPT100gamma/50), 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTSdd*exp(JPT100gamma/50), 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, (JPTSdd*JPTrhoz*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1) - JPTSdd*exp(JPT100gamma/50), 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 1, - JPTSdd*exp(JPT100gamma/50) - (JPTSdd*exp(JPT100gamma/50))/(JPT100DiscRate/100 + 1), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTSdd*exp(JPT100gamma/50), 0;
    0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 - 39/(40*exp(JPT100gamma/100)), -1, 0, 0, -39/(40*exp(JPT100gamma/100)), 0, 1 - 39/(40*exp(JPT100gamma/100)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -39/(40*exp(JPT100gamma/100)), 0, 1 - 39/(40*exp(JPT100gamma/100)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1 - 39/(40*exp(JPT100gamma/100)), -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, -1, -(JPTiotaw/(JPT100DiscRate/100 + 1) + 1)/(1/(JPT100DiscRate/100 + 1) + 1), 0, 0, 0, 0, 0, 0, 0, 0, -((JPTxiw/(JPT100DiscRate/100 + 1) - 1)*(JPTxiw - 1))/(JPTxiw*(5*JPTnu + 1)*(1/(JPT100DiscRate/100 + 1) + 1)), 0, -(JPTiotaw/(JPT100DiscRate/100 + 1) - JPTrhoz/(JPT100DiscRate/100 + 1) + 1)/(1/(JPT100DiscRate/100 + 1) + 1), 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTiotaw/(1/(JPT100DiscRate/100 + 1) + 1), 0, 0, 1/(1/(JPT100DiscRate/100 + 1) + 1), JPTiotaw/(1/(JPT100DiscRate/100 + 1) + 1), 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -((JPTxiw/(JPT100DiscRate/100 + 1) - 1)*(JPTxiw - 1))/(JPTxiw*(5*JPTnu + 1)*(1/(JPT100DiscRate/100 + 1) + 1)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, -JPTnu, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, -(JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/100)*JPTh + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(JPTrhob - 1)*(exp(JPT100gamma/100) - (JPTh*JPTrhob)/(JPT100DiscRate/100 + 1))), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -(JPTh^2/(JPT100DiscRate/100 + 1) + exp(JPT100gamma/100)*JPTh + exp(JPT100gamma/50))/((JPTh - exp(JPT100gamma/100))*(JPTrhob - 1)*(exp(JPT100gamma/100) - (JPTh*JPTrhob)/(JPT100DiscRate/100 + 1))), 0, 0, 0, -JPTnu, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, -JPTphipi*(JPTrhoR - 1), 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1 - JPTrhoR, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    1, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTalpha/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, JPTalpha/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    -39/50, 0, 0, 0, 0, 0, 0, 0, -((39*JPTalpha)/(50*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))) - 39/(50*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))) + (JPTalpha*exp(JPT100gamma/100)*(39/(40*exp(JPT100gamma/100)) - 1))/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))), 0, -JPTalpha/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, (JPTalpha*exp(JPT100gamma/100)*(39/(40*exp(JPT100gamma/100)) - 1))/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, 0, 0, 0, 39/50, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 39/50, 0, 0, 0, 0, 0, -39/50, 0, 0, 0, 0, 0, 0, -((39*JPTalpha)/(50*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))) - 39/(50*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))) + (JPTalpha*exp(JPT100gamma/100)*(39/(40*exp(JPT100gamma/100)) - 1))/((JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))/(1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)))), 0, -JPTalpha/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, (JPTalpha*exp(JPT100gamma/100)*(39/(40*exp(JPT100gamma/100)) - 1))/((1/((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1)) - JPTalpha/((JPTalpha - 1)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))))*(JPTalpha - 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)*((JPTalpha^JPTalpha*(1 - JPTalpha)^(1 - JPTalpha))/((JPTlambdap + 1)*(exp(JPT100gamma/100)*(JPT100DiscRate/100 + 1) - 39/40)^JPTalpha))^(1/(JPTalpha - 1))), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1;
    ];

Gamma4 = [...
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 39/(40*exp(JPT100gamma/100)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 39/(40*exp(JPT100gamma/100)), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhoR, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhoz, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhog, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhow, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -JPTthetaw, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhomu, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhop, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -JPTthetap, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, JPTrhob, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, -1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    ];

Gamma2 = [...
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, JPTsigmaz, 0, 0, 0, 0, 0;
    0, 0, JPTsigmag, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, JPTsigmaw, 0;
    0, 0, 0, JPTsigmamu, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, JPTsigmap, 0, 0;
    0, 0, 0, 0, 0, 0, JPTsigmab;
    JPTsigmamp, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0;
    ];

Gamma3 = eye(52);

cv = (all(Gamma0(1:52,:)==0,2)~=0);
Gamma0(cv,:) = -Gamma1(cv,:);
Gamma1(cv,:) = Gamma4(cv,:);
Gamma3(:,cv) = [];
if ~all(all(Gamma4(~cv,:)==0,2))
  error('Incorrect system reduction')
end

%% Generate matrices for observation equations
if isObsMats>0
   HBar = [...
    JPT100gamma;
    JPT100pi;
    JPT100pi + 100*exp(JPT100gamma/100) + JPT100DiscRate*exp(JPT100gamma/100) - 100;
    ];
   H = [...
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0;
    ];
end

%% Run gensys
if isEvalGensys>0
  [G1,GBar,G2,fmat,fwt,ywt,gev,eu]=vcgensys(Gamma0,Gamma1,GammaBar,...
  Gamma2,Gamma3,'CS',varargin{:});
end
if isObsMats>0 && isEvalGensys>0
if all(GBar(:)==0)
  StateVarBar = zeros(52,1);
else
  StateVarBar = (eye(52)-G1)\GBar;
end
ObsVarBar = HBar + H*StateVarBar;

s00 = zeros(52,1);

[sig00,sig00rc]=lyapcsd(G1,G2*G2');
sig00 = real(sig00);sig00 = (sig00+sig00')/2;
if sig00rc~=0
  fprintf(fid+(fid==1),'Warning: Could not find unconditional variance!\n');
end

end

%% Assign Output
if isObsMats>0
  Mats.ObsEq.HBar = HBar;
  Mats.ObsEq.H = H;
end
if isObsMats>0 && isEvalGensys>0
  Mats.KF.StateVarBar = StateVarBar;
  Mats.KF.ObsVarBar = ObsVarBar;
  Mats.KF.s00 = s00;
  Mats.KF.sig00 = sig00;
  Mats.KF.sig00rc = sig00rc;
end
if isObsMats~=1 && isEvalGensys>0
  Mats.REE.GBar=GBar;
  Mats.REE.G1=G1;
  Mats.REE.G2=G2;
  Mats.REE.eu=eu;
end
if isObsMats~=1 && isEvalGensys~=1
   Mats.StateEq.Gamma0=Gamma0;
   Mats.StateEq.Gamma1=Gamma1;
   Mats.StateEq.GammaBar=GammaBar;
   Mats.StateEq.Gamma2=Gamma2;
   Mats.StateEq.Gamma3=Gamma3;
end
